/**
 * Code released under WTFPL [http://sam.zoy.org/wtfpl/]
 * 
 * @author Arnaud Pichery [http://coderpeon.wordpress.com]
 */
package com.coderpeon.comptree
{
	import flash.display.DisplayObject;
	import flash.events.Event;
	
	import mx.collections.ArrayCollection;
	import mx.collections.Sort;
	import mx.collections.SortField;
	import mx.controls.Alert;
	import mx.controls.Button;
	import mx.controls.Tree;
	import mx.core.Container;
	import mx.core.UIComponent;
	import mx.events.CloseEvent;
	import mx.events.CollectionEvent;
	import mx.events.FlexEvent;
	import mx.managers.PopUpManager;
	import mx.containers.TitleWindow;
	import mx.core.Application;
	import flash.events.KeyboardEvent;
	import mx.managers.FocusManager;
	import flash.events.MouseEvent;
	import mx.events.ListEvent;

	/**
	 * <p>Displays details about a component and its children in a window.</p>
	 * 
	 * <p>To display the window, you call the method <code>show(root: DisplayObject, modal: Boolean)</code>.</p>
	 */
	public class ComponentTreeWnd extends TitleWindow
	{
		// UI component displaying the tree of all components.
		private var _componentTree: Tree;
		
		// UI component displaying the details of a component.
		private var _componentProperties: ComponentProperties;
		
		// Root component of the tree
		private var _root: DisplayObject;
		
		// Indicates whether the window is displayed in a modal mode.
		private var _modal: Boolean;
		
		// Only instance of our window. Non-null when window is displayed, null otherwise.
		private static var _instance: ComponentTreeWnd;
		
		/**
		 * Register a sequence key that will trigger the show of the component tree window.
		 * 
		 * @param root Root of the displayed tree. It cannot be null
		 * @param modal true to display a modal window (default), false to display a modeless window.
		 */
		public static function registerKey(root: DisplayObject, keyCode: uint, ctrlPressed: Boolean = true, altPressed: Boolean = true, shiftPressed: Boolean = false): void {
			root.addEventListener(KeyboardEvent.KEY_DOWN, function(event: KeyboardEvent): void {
				if ((ctrlPressed && !event.ctrlKey) || (altPressed && !event.altKey) || (shiftPressed && !event.shiftKey))
					return;
				
				if (event.keyCode == keyCode) {
					ComponentTreeWnd.show(DisplayObject(event.currentTarget), true);
				}
			});
		}
	
		/**
		 * Displays the tree of the specified DisplayObject (its children, the children of its children, etc.)
		 * 
		 * @param root Root of the displayed tree. If it is null, the current application is used.
		 * @param modal true to display a modal window (default), false to display a modeless window.
		 */
		public static function show(root: DisplayObject = null, modal: Boolean = true): void {
			if (root == null)
			{
				root = DisplayObject(Application.application);
			}
			
			if (_instance == null) {
				_instance = ComponentTreeWnd(PopUpManager.createPopUp(root, ComponentTreeWnd, modal));
				_instance._root = root;
				_instance._modal = modal;
				PopUpManager.centerPopUp(_instance);
			} else if (!_instance._modal) {
				// Modeless window displayed, update the root component and refresh.
				_instance._root = root;
				_instance.initWindow();
			} else {
				// Modal window, do nothing.
			}
		}

	    /**
	     * This method is not intented to be used. Use <code>show</code> method instead.
	     */
		public function ComponentTreeWnd() {
			super();
			
			// properties
			this.layout = "vertical";
			this.title = "Component Tree";
			this.width = 600;
			this.height = 678;
			this.showCloseButton = true;

			// events
			this.addEventListener("close", closeWindow);
			this.addEventListener("creationComplete", onCreationComplete);
			this.addEventListener(KeyboardEvent.KEY_UP, onKeyPressed);
		}
		
		override protected function createChildren(): void {
			super.createChildren();
			
			var vbox: mx.containers.VDividedBox = new mx.containers.VDividedBox();
			vbox.percentWidth = 100.0;
			vbox.percentHeight = 100.0;
			
			_componentTree = new Tree();
			_componentTree.percentWidth = 100.0;
			_componentTree.percentHeight = 60.0;
			_componentTree.addEventListener(ListEvent.CHANGE, onItemSelected);
			_componentTree.iconFunction = getTreeNodeIcon;
			vbox.addChild(_componentTree);
			
			_componentProperties = new ComponentProperties();
			_componentProperties.percentWidth = 100.0;
			_componentProperties.percentHeight = 40.0;
			vbox.addChild(_componentProperties);
			this.addChild(vbox);
			
			var cbar: mx.containers.Box = new mx.containers.HBox();
   			cbar.setStyle("horizontalAlign", "right");
   			cbar.percentWidth = 100.0;
			if (!_modal) {
				// Add refresh button when window is not modal
				var refreshButton: Button = new Button();
				refreshButton.label = "Refresh";
				refreshButton.addEventListener(MouseEvent.CLICK, onRefreshButtonClicked);
				cbar.addChild(refreshButton);
			}
			var closeButton: Button = new Button();
			closeButton.label = "Close";
			closeButton.addEventListener("click", closeWindow);
			cbar.addChild(closeButton);
			this.addChild(cbar);
		}
		
		private function closeWindow(event: Event = null): void {
			PopUpManager.removePopUp(this);
			_instance = null;			
		}
		
		private function onKeyPressed(event: KeyboardEvent): void {
			if (event.keyCode == 27) { // 27 = ESC key
				closeWindow();
			}
		}

		private function onCreationComplete(event: Event): void {
			initWindow();
			focusManager.setFocus(_componentTree);
		}

		private function onRefreshButtonClicked(event: Event): void {
			initWindow();
		}
		
		/**
		 * Fill-in the component tree with the root component.
		 * 
		 * This method is called the first time the window is created, 
		 * and when users click the "refresh" button.
		 */
		private function initWindow(): void {
			// Compute the component tree and display it
			var rootItem: IComponentTreeItem = new ComponentTreeItem(_root);
			var treeNodes: ArrayCollection = new ArrayCollection();
			treeNodes.addItem(rootItem);
			_componentTree.dataProvider = treeNodes;
			_componentTree.validateNow();
			_componentTree.expandItem(rootItem, true);
			_componentProperties.showComponentProperties(null);
		}
		
        private function onItemSelected(event: Event): void {
			var item: ComponentTreeItem = event.currentTarget.selectedItem as ComponentTreeItem;
			var displayObject: DisplayObject = (item == null) ? null : item.displayObject;
			_componentProperties.showComponentProperties(displayObject);
        }
        
		private function getTreeNodeIcon(item:Object):Class{
		   return IComponentTreeItem(item).icon;
		}

		private static function onRegistrationKeyPressed(event: KeyboardEvent): void {
			if (event.ctrlKey && event.shiftKey && event.keyCode == 84) {
					ComponentTreeWnd.show(Application(event.currentTarget), true);
			}
		}
	}
}
